;*****************************************************************************
; Tworzy 16 bitowy kolor z trzech skladowych RGB ( 0 - 63 )
; Wejscie:
;   Al - skladowa R
;   BL - skladowa G
;   CL - skladowa B
; Wyjscie:
;   w ax jest utworzony kolor
;*****************************************************************************
MakeCol16       Proc
        and     ax, 003fh
        shr     ax, 1
        shl     ax, 11
        r_shift = $ - 1

        and     bx, 003fh
        shl     bx, 5
        g_shift = $ - 1

        and     cx, 003fh
        shr     cx, 1
        shl     cx, 0
        b_shift = $ - 1

        or      ax, bx
        or      ax, cx

        ret
MakeCol16       EndP

;*****************************************************************************
; Konwersja obrazka 256 kolorowego na 65536 kolorowy
; Wejscie:
;   ESI - obrazek do konwersji ( 320x200 )
;   EBP - adres palety obrazka
;   EDI - adres, pod ktorym zapisac skonwertowany obrazek
;   EDX - ilosc pixeli do konwersji
; Wyjscie:
;   brak
;*****************************************************************************
ConvertPic      Proc
cp_pet: xor     eax, eax
        xor     ebx, ebx
        lodsb
        mov     bx, ax
        add     ax, ax
        add     ax, bx
        mov     cl, [ ebp + eax + 2]
        mov     bl, [ ebp + eax + 1]
        mov     al, [ ebp + eax ]
        and     cx, 3fh
        and     bx, 3fh
        and     ax, 3fh
        call    MakeCol16
        stosw
        dec     edx
        jnz     cp_pet

        ret
ConvertPic      EndP

;*****************************************************************************
; Wyczyszczenie bufora o wymiarach 320*200*2
; Wejscie:
;   EDI - bufor do wyczyszczenia
; Wyjscie:
;   brak
;*****************************************************************************
Clear_buffer    Proc
        xor     eax, eax
        mov     ecx, 320*200*2/4
        rep     stosd

        ret
Clear_buffer    EndP

;*****************************************************************************
; Wypisanie tekstu do bufora
; Wejscie:
;   EDI - przesuniecie w buforze ( o szerokosci 320 pixeli (640 b) )
;   EBP - adres tekstu do wypisania zakonczonego \0 ( malymi literami !!! )
; Wyjscie:
;   brak
;*****************************************************************************
Write_text      Proc
        add     edi, mem_allocated
        add     edi, Font_buf_ofs
wt_pet_zew:
        mov     esi, mem_allocated
        add     esi, Font_ofs
        xor     eax, eax
        mov     al, [ebp]
        cmp     al, 0
        jz      wt_koniec

        cmp     al, 97
        jnl     nie_literka
        add     esi, 288*17*2
        sub     eax, 32
        jmp     here
nie_literka:
        sub     eax, 97
here:
        mov     ebx, eax                ;\
        add     ebx, eax                ; obliczenie ofsetu znaku, czyli
        shl     eax, 4                  ; (eax - 97) * 22, lub
        add     ebx, eax                ; (eax - 32) * 22
        shr     eax, 2                  ;
        add     ebx, eax                ;/

        mov     eax, 16
wt_pet_wew_y:
        mov     ecx, 10
wt_pet_wew_x:
        mov     dx, [esi + ebx]

        cmp     dx, 0
        jz      nie_rysuj
        mov     [edi], dx
nie_rysuj:

        add     edi, 2
        add     ebx, 2
        dec     ecx
        jnz     wt_pet_wew_x

        add     edi, 640 - 20
        add     ebx, 576 - 20

        dec     eax
        jnz     wt_pet_wew_y

        inc     ebp
        sub     edi, 640*16-20
        jmp     wt_pet_zew

wt_koniec:
        inc     ebp
        mov     text_addy, ebp
        cmp     w [ebp], 0
        jnz     jest_jeszcze_tekst
        mov     text_addy, o texty
jest_jeszcze_tekst:
        ret
Write_text      EndP

;*****************************************************************************
; Stawia wycinek ekranu o wymiarach 128x64 w buforze o wymiarach 320x200
; Wejscie:
;   ESI - adres wycinka
;   EDI - adres bufora
; Wyjscie:
;   brak
;*****************************************************************************
Stamp_hicolor_logo      Proc
        mov     edx, 64
sl_pet_zew:
        mov     ecx, 128*2/4
        rep     movsd

        add     edi, (320 - 128)*2

        dec     edx
        jnz     sl_pet_zew

        ret
Stamp_hicolor_logo      EndP

;*****************************************************************************
; Stawia wycinek ekranu o wymiarach 128x64 w buforze o wymiarach 320x200
; Wejscie:
;   ESI - adres wycinka
;   EDI - adres bufora
; Wyjscie:
;   brak
;*****************************************************************************
Stamp_256_logo  Proc
        mov     edx, 64
sl2_pet_zew:
        mov     ecx, 128
        rep     movsb

        add     edi, 320 - 128

        dec     edx
        jnz     sl2_pet_zew

        ret
Stamp_256_logo  EndP

;*****************************************************************************
; Wpisuje do bufora 256x256 odwrocony wycinek 128x64
; Wejscie:
;   ESI - adres wycinka
;   EDI - adres bufora
; Wyjscie:
;   brak
;*****************************************************************************
Reversed_stamp  Proc
        mov     edx, 64
rs_pet_zew:
        mov     ecx, 128
rs_pet_wew:
        mov     al, [esi + ecx]
        mov     [edi], al

        inc     edi
        dec     ecx
        jnz     rs_pet_wew

        add     edi, 256 - 128
        add     esi, 128

        dec     edx
        jnz     rs_pet_zew

        ret
Reversed_stamp  EndP
