;*****************************************************************************
; mOtzEl's VESA library for EOS ( wersja minimalna do potrzeb intra )
; coded by mOtzEl / cRyOGeN^sUSPENd
;
; Last update: 05.03.98
;*****************************************************************************

;*****************************************************************************
; Definicje struktur
;*****************************************************************************
VbeInfoBlock struc
 VbeSignature        dd   ?             ; VBE Signature
 VbeVersion          dw   ?             ; VBE Version
 OemStringPtr        dd   ?             ; Pointer to OEM String
 Capabilities        dd   ?             ; Capabilities of graphics cont.
 VideoModePtr        dd   ?             ; Pointer to Video Mode List
 TotalMemory         dw   ?             ; Number of 64kb memory blocks
                                        ; Added for VBE 2.0
 OemSoftwareRev      dw   ?             ; VBE implementation Software revision
 OemVendorNamePtr    dd   ?             ; Pointer to Vendor Name String
 OemProductNamePtr   dd   ?             ; Pointer to Product Name String
 OemProductRevPtr    dd   ?             ; Pointer to Product Revision String
 Reserved            db   222 dup (?)   ; Reserved for VBE implementation
                                        ; scratch area
 OemData             db   256 dup (?)   ; Data Area for OEM Strings
VbeInfoBlock ends

ModeInfoBlock   struc
 ; Mandatory information for all VBE revisions
 ModeAttributes      dw ?      ; mode attributes
 WinAAttributes      db ?      ; window A attributes
 WinBAttributes      db ?      ; window B attributes
 WinGranularity      dw ?      ; window granularity
 WinSize             dw ?      ; window size
 WinASegment         dw ?      ; window A start segment
 WinBSegment         dw ?      ; window B start segment
 WinFuncPtr          dd ?      ; pointer to window function
 BytesPerScanLine    dw ?      ; bytes per scan line

 ; Mandatory information for VBE 1.2 and above
 XResolution         dw ?      ; horizontal resolution in pixels or chars
 YResolution         dw ?      ; vertical resolution in pixels or chars
 XCharSize           db ?      ; character cell width in pixels
 YCharSize           db ?      ; character cell height in pixels
 NumberOfPlanes      db ?      ; number of memory planes
 BitsPerPixel        db ?      ; bits per pixel
 NumberOfBanks       db ?      ; number of banks
 MemoryModel         db ?      ; memory model type
 BankSize            db ?      ; bank size in KB
 NumberOfImagePages  db ?      ; number of images
 Reserved1           db ?      ; reserved for page function

 ; Direct Color fields (required for direct/6 and YUV/7 memory models)
 RedMaskSize         db ?      ; size of direct color red mask in bits
 RedFieldPosition    db ?      ; bit position of lsb of red mask
 GreenMaskSize       db ?      ; size of direct color green mask in bits
 GreenFieldPosition  db ?      ; bit position of lsb of green mask
 BlueMaskSize        db ?      ; size of direct color blue mask in bits
 BlueFieldPosition   db ?      ; bit position of lsb of blue mask
 RsvdMaskSize        db ?      ; size of direct color reserved mask in bits
 RsvdFieldPosition   db ?      ; bit position of lsb of reserved mask
 DirectColorModeInfo db ?      ; direct color mode attributes

 ; Mandatory information for VBE 2.0 and above
 PhysBasePtr         dd ?      ; physical address for flat frame buffer
 OffScreenMemOffset  dd ?      ; pointer to start of off screen memory
 OffScreenMemSize    dw ?      ; amount of off screen memory in 1k units
 Reserved2           db 206 dup (?)  ; remainder of ModeInfoBlock
ModeInfoBlock ends

;*****************************************************************************
; Definicje zmiennych
;*****************************************************************************
align 4
VBE_CardInfo            VbeInfoBlock <>   ; db 512 dup(0)
VBE_ModeInfo            ModeInfoBlock <>  ; db 256 dup(0)
VBE_LFBAddress          dd 0
VBE_PageSize            dd 0
virtual_screen          dd 0
virt_handle             dd -1
VBE_ModesTable          dw 512*4 dup (0)  ; tryb, x, y, bpp
VBE_BytesPerLine        dw 0
VBE_NumberOfModes       dw 0
VBE_VideoPages          db 0
VBE_Error               db 0

;*****************************************************************************
; Detekcja Vesy :
; Wejscie:
;   brak
; Wyjscie:
;   jesli brak VESY to ustawia flage przeniesienia i zmienna VBE_Error
;   jesli jest to wypelnia strukture VBE_CardInfo i zmienne VBE_TotalMemory,
;    VBE_Version oraz tablice trybow ( VBE_ModesTable )
;*****************************************************************************
VBE_Detect      Proc
        mov     edi, o VBE_CardInfo
        mov     eax, '2BEV'
        stosd

        Init_es_di VBE_CardInfo
        mov     ax, 4f00h
        DosInt  10h

        cmp     ax, 004fh
        je      d_ok_vesa2
        stc
        ret

d_ok_vesa2:
        mov     esi, o VBE_CardInfo
        cmp     [esi.VbeSignature], 'ASEV'
        je      d_ok_vesa3
        stc
        ret

d_ok_vesa3:
        mov     esi, o VBE_CardInfo

        xor     eax, eax
        xor     edx, edx
        mov     ax, w [esi.VideoModePtr]
        mov     dx, w [esi.VideoModePtr+2]
        shl     edx, 4
        add     edx, eax
        sub     edx, [Code32_Addr]      ; w edx mamy wskaznik do listy trybow

; pobranie informacji o wszystkich trybach video
        mov     edi, o VBE_ModesTable
        mov     esi, edx
        xor     edx, edx
getmodesloop:
        lodsw
        cmp     ax, 0ffffh
        jz      allmodes

        pushad
        mov     cx, ax
        call    VBE_GetModeInfo
        popad
        jc      getmodesloop

        mov     ebx, o VBE_ModeInfo

        mov     cl, [ebx.MemoryModel]
        cmp     cl, 4                   ; tryby 256 kolorowe
        jnz     nie_256
        jmp     tryb_graficzny
nie_256:
        cmp     cl, 6                   ; tryby 15/16/24 bitowe
        jnz     getmodesloop

tryb_graficzny:
        stosw

        mov     ax, [ebx.XResolution]
        stosw

        mov     ax, [ebx.YResolution]
        stosw

        xor     ax, ax
        mov     al, [ebx.BitsPerPixel]
        stosw

        inc     edx

        jmp     getmodesloop

allmodes:
        mov     VBE_NumberOfModes, dx

        clc
        ret
VBE_Detect      EndP


;*****************************************************************************
; Dealokacja pamieci i rozmapowanie LFB
; Wejscie:
;   brak
; Wyjscie:
;   brak
;*****************************************************************************
VBE_Close       Proc
        call    VBE_UnMapLFB

        cmp     virt_handle, -1
        jz      brak_alokacji

        mov     ah, Deallocate_Memory_Handle
        mov     edi, virt_handle
        Int_EOS

brak_alokacji:
        mov     virt_handle, -1

        ret
VBE_Close       EndP

;*****************************************************************************
; Wlaczenie trybu video o podanej rozdzielczosci i ilosci kolorow
; Wejscie:
;   AX - rozdzielczosc X
;   BX - rozdzielczosc Y
;   DX - BPP ( 8, 15, 16, 24, 32 )
; Wyjscie:
;   jesli sie nie powiodlo to ustawia flage przeniesienia i zmienna VBE_Error
;   jesli sie powiodlo to wypelniona jest struktura VBE_ModeInfo i ustawione
;    zmienne VBE_BytesPerLine, VBE_VideoPages, VBE_PageSize
;*****************************************************************************
VBE_InitMode    Proc
        xor     ecx, ecx
        mov     esi, o VBE_ModesTable
        mov     cx, VBE_NumberOfModes

initloop:
        cmp     ax, [esi+2]
        jnz     badmode
        cmp     bx, [esi+4]
        jnz     badmode
        cmp     dx, [esi+6]
        jz      modefound
badmode:
        add     esi, 8
        loop    initloop

        stc
        ret

modefound:
        mov     cx, [esi]
        push    cx
        call    VBE_GetModeInfo
        jnc     i_dal
        stc
        ret
i_dal:
        mov     esi, o VBE_ModeInfo

        mov     ax, [esi.ModeAttributes]
        and     ax, 80h
        jnz     jest_lfb
        pop     cx                      ; zdjecie ze stosu polozonego numeru trybu
                                        ; bo adres powrotu sie nie bedzie zgadzal

        stc
        ret

jest_lfb:
        mov     esi, o VBE_ModeInfo
        mov     al, [esi.RedFieldPosition]
        mov     b r_shift, al
        mov     al, [esi.GreenFieldPosition]
        mov     b g_shift, al
        mov     al, [esi.BlueFieldPosition]
        mov     b b_shift, al

        xor     eax, eax
        xor     ebx, ebx

        mov     ax, [esi.BytesPerScanLine]
        mov     VBE_BytesPerLine, ax

        mov     bx, [esi.YResolution]
        mul     bx
        shl     edx, 16
        mov     dx, ax
        mov     VBE_PageSize, edx

        mov     al, [esi.NumberOfImagePages]
        mov     VBE_VideoPages, al

        call    VBE_MapLFB
        jc      map_error

        mov     ah, Allocate_Memory_Handle
        mov     edx, VBE_PageSize
        Int_EOS
        jnc     ok_mem
        jmp     error

ok_mem:
        mov     virtual_screen, edx
        mov     virt_handle, edi

        pop     bx
        call    VBE_SetMode
        jc      error

        clc
        ret

map_error:
        pop     cx                      ; zdjecie ze stosu polozonego numeru trybu
                                        ; bo adres powrotu sie nie bedzie zgadzal

error:
        stc
        ret

VBE_InitMode    EndP

;*****************************************************************************
; Wlaczenie mapowania LFB
; Wejscie:
;   musi byc ustawiona zmienna VBE_TotalMemory ( funkcja VBE_Detect )
; Wyjscie:
;   jesli sie nie powiodlo to ustawia flage przeniesienia
;   jesli sie powiodlo to ustawia zmienna VBE_LFBAddress
;*****************************************************************************
VBE_MapLFB      Proc
        cmp     VBE_LFBAddress, 0
        jz      mapuj
        call    VBE_UnMapLFB

mapuj:
        mov     ah, Physical_Address_Mapping
        mov     ecx, 256*1024
        mov     esi, [esi.PhysBasePtr]
        Int_EOS
        jc      blad_mapowania          ; blad mapowania - brak pamieci
        sub     esi, [Code32_Addr]
        mov     VBE_LFBAddress, esi

        clc
        ret

blad_mapowania:

        stc
        ret
VBE_MapLFB      EndP

;*****************************************************************************
; Wylaczenie mapowania LFB
; Wejscie:
;   brak
; Wyjscie:
;   ustawia zmienna VBE_LFBAddress na 0
;*****************************************************************************
VBE_UnMapLFB    Proc
        cmp     VBE_LFBAddress, 0
        jnz     bylo_mapowanie
        ret

bylo_mapowanie:
        mov     ah, Free_Physical_Address_Mapping
        mov     esi, VBE_LFBAddress
        add     esi, [Code32_Addr]
        Int_EOS
        mov     VBE_LFBAddress, 0

        ret
VBE_UnMapLFB    EndP

;*****************************************************************************
; Pobranie informacji o trybie:
; Wejscie:
;   CX - numer trybu
; Wyjscie:
;   jesli brak ustawia flage przeniesienia i zmienna VBE_Error
;   jesli jest to wypelnia strukture VBE_ModeInfo
;*****************************************************************************
VBE_GetModeInfo Proc
        Init_es_di VBE_ModeInfo
        mov     ax, 4f01h
        DosInt  10h

        cmp     ax, 004fh
        je      g_ok_vesa2
        stc
        ret

g_ok_vesa2:
        mov     esi, o VBE_ModeInfo
        xor     eax, eax
        mov     ax, [esi.ModeAttributes]
        and     ax, 01h
        jnz     g_ok_vesa3
        stc
        ret

g_ok_vesa3:

        clc
        ret

VBE_GetModeInfo EndP

;*****************************************************************************
; Ustawienie trybu graficznego:
; Wejscie:
;   BX - numer trybu do ustawienia ( 4000h jest dodawane automatycznie )
; Wyjscie:
;   jesli blad ustawia flage przeniesienia i zmienna VBE_Error
;*****************************************************************************
VBE_SetMode     Proc
        add     bx, 4000h

        mov     ax, 4f02h
        DosInt  10h

        cmp     ax, 004fh
        je      s_ok_vesa2
        stc
        ret

s_ok_vesa2:

        clc
        ret
VBE_SetMode     EndP

;*****************************************************************************
; Ustawia adres poczatku obrazu:
; Wejscie:
;   AX = 0  - operacja wykona sie natychmiast
;   AX = 80 - operacja wykona sie dopiero podczas powrotu pionowego
;   CX - pierwszy pixel w lini, ktory ma byc wyswietlony
;   DX - pierwsza linia, ktora ma byc wyswietlona
; Wyjscie:
;   jesli blad ustawia flage przeniesienia
;   CX, DX - zwraca dane jak na wejsciu
;*****************************************************************************
VBE_SetDisplayStart      Proc
        xor     bx, bx
        add     bx, ax
        mov     ax, 4f07h
        DosInt  10h

        cmp     ax, 004fh
        je      se_ok_vesa2
        stc
        ret

se_ok_vesa2:

        clc
        ret
VBE_SetDisplayStart     EndP

;*****************************************************************************
; Kopiuje virtual_screen na ekran uzywajac LFB
; Wejscie:
;   ECX - wielkosc obszaru do skopiowania / 4
; Wyjscie:
;   brak
;*****************************************************************************
VBE_SwapPages   Macro
        mov     edi, VBE_LFBAddress
        mov     esi, virtual_screen
        rep     movsd
EndM
